<?php

namespace backend\modules\location\models\search;

use yii\base\Model;
use yii\data\ActiveDataProvider;
use yii\db\ActiveQuery;

use thread\app\model\interfaces\search\BaseBackendSearchModel;

use backend\modules\location\Location as MainModule;
use backend\modules\location\models\Region as MainModel;
use common\modules\location\models\RegionLang as MainModelLang;


/**
 * Class Region
 * @package backend\modules\location\models\search
 * @author Alla Kuzmenko
 * @copyright (c), Thread
 */
class Region extends MainModel implements BaseBackendSearchModel
{
    public $title;

    /**
     * @return array
     */
    public function rules()
    {
        return [
            [['title'], 'string', 'max' => 255]
        ];
    }

    /**
     *
     * @return array
     */
    public function scenarios()
    {
        return Model::scenarios();
    }

    /**
     * @param ActiveQuery $query
     * @param array $params
     * @return ActiveDataProvider
     */
    public function baseSearch($query, $params)
    {
        /** @var MainModule $module */
        $module = $this->getModule();
        $dataProvider = new ActiveDataProvider(
            [
                'query' => $query,
                'pagination' => [
                    'pageSize' => $module->itemOnPage
                ],
                'sort' => [
                    'defaultOrder' => [
                        'position' => SORT_ASC
                    ]
                ]
            ]
        );

        if (!($this->load($params) && $this->validate())) {
            return $dataProvider;
        }

        $query->andFilterWhere(['like', 'published', $this->published])
            ->andFilterWhere(['like', 'deleted', $this->deleted]);
        $query->andFilterWhere(['like', MainModelLang::tableName() . '.title', $this->title]);


        return $dataProvider;
    }

    /**
     * Search
     *
     * @param array $params
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = MainModel::find()->joinWith(['lang'])->orderBy('position')->undeleted();
        return $this->baseSearch($query, $params);
    }

    /**
     * @param array $params
     * @return ActiveDataProvider
     */
    public function trash($params)
    {
        $query = MainModel::find()->joinWith(['lang'])->orderBy('position')->deleted();
        return $this->baseSearch($query, $params);
    }
}
