<?php

namespace backend\modules\map\models\search;

use yii\base\Model;
use yii\data\ActiveDataProvider;
use yii\db\ActiveQuery;

use thread\app\model\interfaces\search\BaseBackendSearchModel;

use backend\modules\map\Map as MainModule;
use backend\modules\map\models\{
    Item as MainModel, ItemLang as MainLangModel
};


/**
 * Class Item
 * @package backend\modules\map\models\search
 * @author Alla Kuzmenko
 * @copyright (c), Thread
 */
class Item extends MainModel implements BaseBackendSearchModel
{
    public $title;

    public function rules()
    {
        return [
            [['title', 'type'], 'string'],
            [['type_id', 'area_id'], 'integer'],
        ];
    }

    /**
     *
     * @return array
     */
    public function scenarios()
    {
        return Model::scenarios();
    }

    /**
     * @param ActiveQuery $query
     * @param array $params
     * @return ActiveDataProvider
     */
    public function baseSearch($query, $params)
    {
        /** @var MainModule $module */
        $module = $this->getModule();
        $dataProvider = new ActiveDataProvider(
            [
                'query' => $query,
                'pagination' => [
                    'pageSize' => $module->itemOnPage
                ],
                'sort' => [
                    'defaultOrder' => [
                        'id' => SORT_DESC
                    ]
                ]
            ]
        );

        if (!($this->load($params) && $this->validate())) {
            return $dataProvider;
        }

        $query->andFilterWhere(['=', 'published', $this->published])
            ->andFilterWhere(['=', 'deleted', $this->deleted])
            ->andFilterWhere(['=', 'type_id', $this->type_id])
            ->andFilterWhere(['=', 'area_id', $this->area_id]);

        $query->andFilterWhere(['like', MainLangModel::tableName() . '.title', $this->title]);
        $query->andFilterWhere(
            [
                'type' => $this->type,
            ]
        );


        return $dataProvider;
    }

    /**
     * Search
     *
     * @param array $params
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = MainModel::find()->joinWith(['lang'])->undeleted();
        return $this->baseSearch($query, $params);
    }

    /**
     * @param array $params
     * @return ActiveDataProvider
     */
    public function trash($params)
    {
        $query = MainModel::find()->joinWith(['lang'])->deleted();
        return $this->baseSearch($query, $params);
    }
}
