<?php

use yii\helpers\Html;
use backend\modules\map\models\{
    Area, Type
};
use backend\modules\location\models\Region;
use backend\modules\map\models\Item;
use backend\themes\defaults\widgets\mapInputWidget\widgets\MapInputWidget;

/**
 * @var \backend\modules\map\models\Item $model
 **/
?>
<div class="row form-group">
    <?= Html::beginTag('div', ['class' => 'col-md-11'])
    . $form->text_line_lang($modelLang, 'title')
    . Html::endTag('div')
    . Html::beginTag('div', ['class' => 'col-md-12'])
    . $form->text_line($model, 'alias')
    . Html::endTag('div')
    ?>
</div>
<div class="row form-group">
    <?= Html::beginTag('div', ['class' => 'col-md-6'])
    . $form->field($model, 'type_id')->dropDownList(Type::getDropDownList($model['type']))
    . Html::endTag('div')
    . Html::beginTag('div', ['class' => 'col-md-6'])
    . $form->field($model, 'area_id')->dropDownList(Area::getDropDownList($model['type']))
    . Html::endTag('div')
    ?>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($model, 'region_id')->dropDownList(Region::getDropDownList()) ?>
    </div>
    <div class="col-md-6">
        <?= $form->field($modelLang, 'address')->textInput(['maxlength' => true])->label($modelLang->getAttributeLabel('address') . ' - <span style="color: red"></span>') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->field($model, 'coordinates')->widget(MapInputWidget::class,
            [
                'key' => 'AIzaSyCUmy-9mReTZS50UTtYAdxev22BUXYk_ig',
                'latitude' => 50,
                'longitude' => 32,
                'zoom' => 6,
                'height' => '600px',
            ]);
        ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($model, 'electric_power')->textInput() ?>
    </div>
    <div class="col-md-6">
        <?= $form->field($model, 'heat_capacity')->textInput() ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($modelLang, 'initiator')->textInput(['maxlength' => true]) ?>
    </div>
    <div class="col-md-6">
        <?= $form->field($model, 'type_of_promoter')->dropDownList(Item::type_of_promoterKeyRange()) ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'contact_details_of_project_promoter') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($model, 'date_of_entry')->textInput(['maxlength' => true]) ?>
    </div>
    <div class="col-md-6">
        <?= $form->field($model, 'current_stage_of_project_development')->dropDownList(Item::current_stage_of_project_developmentKeyRange()) ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'description') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($modelLang, 'annual_power_production')->textInput(['maxlength' => true]) ?>
    </div>
    <div class="col-md-6">
        <?= $form->field($modelLang, 'annual_heat_production')->textInput(['maxlength' => true]) ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($modelLang, 'power_price')->textInput(['maxlength' => true]) ?>
    </div>
    <div class="col-md-6">
        <?= $form->field($modelLang, 'heat_price')->textInput(['maxlength' => true]) ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($modelLang, 'project_implementation_period')->textInput(['maxlength' => true]) ?>
    </div>
    <div class="col-md-6">

    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'project_documentation') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'investment_schedule') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-sm-12">
        <?php $drop = Yii::$app->language == 'en-EN' ?
            \frontend\modules\map\models\ItemLang::licensesRangeListEN()
            :
            \frontend\modules\map\models\ItemLang::licensesRangeListUA() ?>
        <?= $form->field($modelLang, 'licenses')->widget(\kartik\select2\Select2::classname(), [
            'data' => $drop,
            'options' => [
                'value' => explode(',', $modelLang['licenses']),
                'placeholder' => '',
                'multiple' => true,
                'allowClear' => true
            ],
        ]) ?>
    </div>
    <div class="col-sm-12">
        <?= $form->field($modelLang, 'licenses_other')->textarea() ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'estimated_return_on_investment') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'technical_parameters') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($model, 'investment')->textInput(['maxlength' => true]) ?>
    </div>
    <div class="col-md-6">

    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'forms_cooperation_with_investor') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-6">
        <?= $form->field($model, 'share_of_project_equity_offered') ?>
    </div>
    <div class="col-md-6">
        <?= $form->field($modelLang, 'expected_financial_return') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'financing_sources') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'financing_debt') ?>
    </div>
</div>
<div class="row form-group">
    <div class="col-md-12">
        <?= $form->text_lang($modelLang, 'expenditure_additional_infrastructure') ?>
    </div>
</div>

<div class="row form-group">
    <?= $form->field($model, 'gallery_file')->fileSeveral(
        ['initialPreview' => $model->getGalleryFileLink(), 'maxFileCount' => 25],
        ['maxFileCount' => 25])
        ->label($model->getAttributeLabel('gallery_file') . ' 590X390 (jpg, png)') ?>
</div>

<div class="row form-group">
    <?php
    echo Html::beginTag('div', ['class' => 'col-md-6'])
        . $form->field($model, 'web_site')->textInput(['maxlength' => true])
        . Html::endTag('div')
        . Html::beginTag('div', ['class' => 'col-md-6'])
        . $form->field($model, 'youtube_link')->textInput(['maxlength' => true])
        . Html::endTag('div');
    if ($model->isNewRecord) {
        echo $form->field($model, 'type')->hiddenInput()->label('');
        echo $form->field($model, 'user_id')->hiddenInput(['value' => Yii::$app->getUser()->getId()])->label('');
    }
    ?>
</div>

<div class="row form-group">
    <?php
    echo Html::beginTag('div', ['class' => 'col-md-2'])
        . $form->switcher($model, 'published')
        . Html::endTag('div');
    ?>
</div>
