<?php

namespace backend\modules\news\modules\review\models\search;

use Yii;
use yii\data\ActiveDataProvider;
use yii\base\Model;
//
use thread\app\base\models\query\ActiveQuery;
use thread\app\model\interfaces\search\BaseBackendSearchModel;
//
use backend\modules\news\News as ParentModule;
use backend\modules\news\modules\review\models\{
    Review as ParentModel
};

/**
 * Class Review
 *
 * @package backend\modules\news\modules\review\models\search
 * @author FilamentV <vortex.filament@gmail.com>
 * @copyright (c), Thread
 */
class Review extends ParentModel implements BaseBackendSearchModel
{
    /**
     * @return array
     */
    public function rules()
    {
        return [
            [['title'], 'string', 'max' => 255],
            [['article_id', 'review_id', 'member_id'], 'integer'],
            [['published'], 'in', 'range' => array_keys(self::statusKeyRange())],
        ];
    }

    /**
     *
     * @return array
     */
    public function scenarios()
    {
        return Model::scenarios();
    }

    /**
     * @param ActiveQuery $query
     * @param array $params
     * @return ActiveDataProvider
     */
    public function baseSearch($query, $params)
    {
        /** @var ParentModule $module */
        $module = Yii::$app->getModule('news');
        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'pagination' => [
                'pageSize' => $module->itemOnPage
            ],
            'sort' => [
                'defaultOrder' => [
                    'created_at' => SORT_DESC,
                    'published' => SORT_ASC,
                ]
            ]
        ]);

        if (!($this->load($params) && $this->validate())) {
            return $dataProvider;
        }

        $query->andFilterWhere(['like', 'product_id', $this->product_id]);
        $query->andFilterWhere(['like', 'member_id', $this->member_id]);
        $query->andFilterWhere(['like', 'title', $this->title]);
        $query->andFilterWhere(['like', 'published', $this->published]);

        return $dataProvider;
    }

    /**
     * @param array $params
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = ParentModel::find()->undeleted();
        return $this->baseSearch($query, $params);
    }

    /**
     * @param array $params
     * @return ActiveDataProvider
     */
    public function trash($params)
    {
        $query = ParentModel::find()->deleted();
        return $this->baseSearch($query, $params);
    }

    /**
     * @param $params
     * @param $article_id
     * @return ActiveDataProvider
     */
    public function search_article_id($params, $article_id)
    {
        $query = ParentModel::find()->undeleted()->article_id($article_id);
        return $this->baseSearch($query, $params);
    }

    /**
     * @param $params
     * @param $article_id
     * @return ActiveDataProvider
     */
    public function trash_article_id($params, $article_id)
    {
        $query = ParentModel::find()->deleted()->article_id($article_id);
        return $this->baseSearch($query, $params);
    }
}
