<?php

namespace backend\modules\seo\modules\modellink\controllers;

use Yii;
use yii\helpers\{
    Url, ArrayHelper
};
//
use thread\app\base\controllers\BackendController;
use thread\actions\{
    Create, Update, AttributeSwitch
};
//
use backend\modules\seo\modules\modellink\models\{
    ModellinkImage, search\ModellinkImage as filterModel
};

/**
 * Class ModellinkImageController
 *
 * @package backend\modules\seo\modules\modellink\controllers
 * @author FilamentV <vortex.filament@gmail.com>
 * @copyright (c), Thread
 */
class ModellinkImageController extends BackendController
{
    public $model = ModellinkImage::class;
    public $filterModel = filterModel::class;
    public $title = 'ModellinkImage';
    public $name = 'modellinkimage';
    public $model_id = 0;
    public $model_key = '';

    /**
     * @return mixed
     * @throws \Exception
     */
    public function init()
    {
        $model_id = Yii::$app->getRequest()->get('model_id');
        $model_key = Yii::$app->getRequest()->get('model_key');

        if ($model_id !== null) {
            $this->setSessionModelId($model_id);
            $this->setSessionModelKey($model_key);
        }
        $this->model_id = $this->getSessionModelId();
        $this->model_key = $this->getSessionModelKey();
        return parent::init();
    }

    /**
     * @param $model_id
     */
    public function setSessionModelId($model_id)
    {
        Yii::$app->getSession()->set('ModellinkImageController-model_id', $model_id);
    }

    /**
     * @return mixed
     */
    public function getSessionModelId()
    {
        return Yii::$app->getSession()->get('ModellinkImageController-model_id', 0);
    }

    /**
     * @param $model_key
     */
    public function setSessionModelKey($model_key)
    {
        Yii::$app->getSession()->set('ModellinkImageController-model_key', $model_key);
    }

    /**
     * @return mixed
     */
    public function getSessionModelKey()
    {
        return Yii::$app->getSession()->get('ModellinkImageController-model_key', '1');
    }

    /**
     * @return array
     */
    public function actions()
    {

        $redirect = function () {
            return Url::to(['list', 'model_id' => $this->model_id, 'model_key' => $this->model_key]);
        };

        $redirect_upd = function () {
            return ($_POST['save_and_exit']) ?
                Url::to(['list', 'model_id' => $this->model_id, 'model_key' => $this->model_key])
                : Url::to(['update', 'id' => $this->action->getModel()->id, 'model_id' => $this->model_id, 'model_key' => $this->model_key]);
        };

        return ArrayHelper::merge(parent::actions(), [
            'list' => [
                'layout' => 'list-modellink-image',
            ],
            'trash' => [
                'layout' => 'list-modellink-image-crud',
            ],
            'create' => [
                'class' => Create::class,
                'redirect' => $redirect_upd,
                'layout' => 'list-modellink-image-crud',
            ],
            'update' => [
                'class' => Update::class,
                'redirect' => $redirect_upd,
                'layout' => 'list-modellink-image-crud',
            ],
            'delete' => [
                'redirect' => $redirect,
            ],
            'add_to_sitemap' => [
                'class' => AttributeSwitch::class,
                'modelClass' => $this->model,
                'attribute' => 'add_to_sitemap',
                'redirect' => $this->defaultAction,
            ],
            'dissallow_in_robotstxt' => [
                'class' => AttributeSwitch::class,
                'modelClass' => $this->model,
                'attribute' => 'dissallow_in_robotstxt',
                'redirect' => $this->defaultAction,
            ],
        ]);
    }
}