<?php

namespace backend\modules\seo\modules\modellink\models\search;

use Yii;
use yii\data\ActiveDataProvider;
use yii\base\Model;
//
use thread\app\base\models\query\ActiveQuery;
use thread\app\model\interfaces\search\BaseBackendSearchModel;
//
use backend\modules\seo\modules\modellink\Modellink as ParentModule;
use backend\modules\seo\modules\modellink\models\{
    ModellinkImage as ParentModel
};

/**
 * Class ModellinkImage
 *
 * @package backend\modules\seo\modules\modellink\models\search
 * @author FilamentV <vortex.filament@gmail.com>
 * @copyright (c), Thread
 */
class ModellinkImage extends ParentModel implements BaseBackendSearchModel
{
    /**
     * @return array
     */
    public function rules()
    {
        return [
            [['model_id'], 'integer'],
            [['model_key'], 'string', 'max' => 40],
            [['image_loc'], 'string', 'max' => 512],
            [['published'], 'in', 'range' => array_keys(self::statusKeyRange())],
        ];
    }

    /**
     *
     * @return array
     */
    public function scenarios()
    {
        return Model::scenarios();
    }

    /**
     * @param ActiveQuery $query
     * @param array $params
     * @return ActiveDataProvider
     */
    public function baseSearch($query, $params)
    {
        /** @var ParentModule $module */
        $module = Yii::$app->getModule('seo');
        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'pagination' => [
                'pageSize' => $module->itemOnPage
            ],
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC
                ]
            ]
        ]);

        if (!($this->load($params) && $this->validate())) {
            return $dataProvider;
        }

        $query->andFilterWhere(['like', 'image_loc', $this->image_loc])
            ->andFilterWhere(['like', 'model_key', $this->model_key])
            ->andFilterWhere(['=', 'published', $this->published]);

        return $dataProvider;
    }

    /**
     * @param array $params
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = ParentModel::find()->undeleted();
        return $this->baseSearch($query, $params);
    }

    /**
     * @param array $params
     * @return ActiveDataProvider
     */
    public function trash($params)
    {
        $query = ParentModel::find()->deleted();
        return $this->baseSearch($query, $params);
    }

    /**
     * @param $params
     * @param $model_id
     * @param $model_key
     * @return ActiveDataProvider
     */
    public function search_model_id_and_key($params, $model_id, $model_key)
    {
        $query = ParentModel::find()->undeleted()->currency_language()->model_key($model_key)->model_id($model_id);
        return $this->baseSearch($query, $params);
    }

    /**
     * @param $params
     * @param $model_id
     * @param $model_key
     * @return ActiveDataProvider
     */
    public function trash_model_id_and_key($params, $model_id, $model_key)
    {
        $query = ParentModel::find()->deleted()->currency_language()->model_key($model_key)->model_id($model_id);
        return $this->baseSearch($query, $params);
    }
}
