<?php

namespace common\modules\map\models;

use Yii;

use thread\app\base\models\ActiveRecord;
use thread\app\base\module\abstracts\Module;
use common\modules\map\Map as MainModule;


/**
 * Class Type
 *
 * @property integer id
 * @property string type_map
 * @property string color
 * @property integer position
 * @property integer created_at
 * @property integer updated_at
 * @property boolean published
 * @property boolean deleted
 * @author Alla Kuzmenko
 * @package common\modules\map\models
 */
class Type extends ActiveRecord
{
    /**
     *
     */
    const moduleName = 'map';


    /**
     * @return null|object|string
     */
    public static function getDb()
    {
        return MainModule::getDb();
    }

    /**
     * @return string
     */
    public static function tableName()
    {
        return '{{%map_type}}';
    }

    /**
     * @return array
     */
    public function rules()
    {
        return [
            [['type_map', 'color'], 'required'],
            [['color'], 'string', 'max' => 1024],
            [['type_map'], 'in', 'range' => array_keys(MainModule::typeRange())],
            [['published', 'deleted'], 'in', 'range' => array_keys(static::statusKeyRange())],
            [['position'], 'default', 'value' => 0],
            [['published'], 'default', 'value' => self::STATUS_KEY_ON]

        ];
    }

    /**
     * @return array
     */
    public function scenarios()
    {
        return [
            'published' => ['published'],
            'deleted' => ['deleted'],
            'position' => ['position'],
            'backend' => [
                'color',
                'type_map',
                'position',
                'published',
                'deleted',
            ],
        ];
    }

    /**
     * @return array
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'color' => Yii::t(self::moduleName, 'color'),
            'type_map' => Yii::t(self::moduleName, 'Type'),
            'position' => Yii::t('app', 'Position'),
            'created_at' => Yii::t('app', 'Create time'),
            'updated_at' => Yii::t('app', 'Update time'),
            'published' => Yii::t('app', 'Published'),
            'deleted' => Yii::t('app', 'Deleted'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLang()
    {
        return $this->hasOne(TypeLang::class, ['rid' => 'id']);
    }

    /**
     * @return Module|null the module instance, `null` if the module does not exist.
     */

    /**
     * @return null|\yii\base\Module
     */
    public static function getModule()
    {
        return Yii::$app->getModule(self::moduleName);
    }

}
