<?php

namespace frontend\modules\calendar\models;

use Yii;
use yii\helpers\Url;
//
use thread\app\model\interfaces\BaseFrontModel;
use thread\modules\seo\interfaces\{
    SeoFrontModel
};

/**
 * Class Article
 *
 * @package frontend\modules\calendar\models
 * @author FilamentV <vortex.filament@gmail.com>
 * @copyright (c), Thread
 */
class Article extends \common\modules\calendar\models\Article implements BaseFrontModel, SeoFrontModel
{

    /**
     *
     * @return array
     */
    public function behaviors()
    {
        return [];
    }

    /**
     *
     * @return array
     */
    public function scenarios()
    {
        return [];
    }

    /**
     *
     * @return array
     */
    public function attributeLabels()
    {
        return [];
    }

    /**
     *
     * @return array
     */
    public function rules()
    {
        return [];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLang()
    {
        return $this->hasOne(ArticleLang::class, ['rid' => 'id']);
    }

    /**
     * @return mixed
     */
    public static function find()
    {
        return parent::find()->enabled()->innerJoinWith(["lang"])->orderBy([self::tableName() . '.published_time' => SORT_DESC]);
    }

    /**
     * @return mixed
     */
    public static function findByParams()
    {
        $get = Yii::$app->request->get();
        $condition = false;
        $q = self::find();
        if (isset($get['year']) && ($get['year'])) {
            $condition = ['from' => '1.1.' . $get['year'], 'to' => '31.12.' . $get['year']];
        }
        if (isset($get['month']) && ($get['month'])) {
            $condition = ['from' => '1.' . $get['month'] . '.' . $get['year'], 'to' => '31.' . $get['month'] . '.' . $get['year']];
        }
        if ($condition) {
            $q->andWhere(['BETWEEN', 'published_time', (int)strtotime($condition['from']), (int)strtotime($condition['to'])]);
        }
        return $q;
    }

    /**
     * @return array
     */
    public static function getMinMaxDate()
    {
        return self::getDb()->createCommand('SELECT MIN(published_time) as min , MAX(published_time) as max FROM ' . self::tableName())->queryOne();
    }

    /**
     * @return mixed
     */
    public static function findBase()
    {
        return self::find();
    }

    /**
     * @return mixed
     */
    public static function findSeo()
    {
        return self::find();
    }

    /**
     * @return mixed
     */
    public static function findSeoNews()
    {
        return self::find();
    }

    /**
     * @return mixed
     */
    public static function findSeoImage()
    {
        return self::find();
    }

    /**
     * @param $id
     * @return mixed
     */
    public static function findById($id)
    {
        return self::find()->byId($id);
    }

    /**
     * @param $alias
     * @return mixed
     */
    public static function findByAlias($alias)
    {
        return self::find()->byAlias($alias);
    }

    /**
     * @param $id
     * @return mixed
     */
    public static function getById($id)
    {
        return self::findById($id)->one();
    }

    /**
     * @param $alias
     * @return mixed
     */
    public static function getByAlias($alias)
    {
        return self::findByAlias($alias)->one();
    }

    /**
     * @return mixed
     */
    public static function getAllWithLabel()
    {
        return self::find()->all();
    }

    /**
     *
     * @return string
     */
    public function getUrl($scheme = false)
    {
        return Url::toRoute(['/calendar/article/index', 'alias' => $this->alias], $scheme);
    }

}
