<?php

use yii\helpers\Html;
use thread\app\bootstrap\ActiveForm;
use backend\modules\map\models\{
    Area, Type
};
use backend\themes\defaults\widgets\mapInputWidget\widgets\MapInputWidget;
use frontend\themes\defaults\assets\AppAsset;
use frontend\modules\map\models\Item;

?>
<div style="position: relative; margin-top: 20px;">
    <?php

    $bundle = AppAsset::register($this);

    $form = ActiveForm::begin();
    if ($model->isNewRecord) {
        $model->type = 'project';
        $model->user_id = Yii::$app->getUser()->getId();
        echo $form->field($model, 'type')->hiddenInput()->label(false);
    }
    ?>
    <?= Html::tag('h1', Yii::t('front', 'Add project'), [
        'class' => 'title-1 reset-margin-top'
    ]) ?>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'title') ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-6">
            <?= $form->field($model, 'type_id')->dropDownList(Type::getDropDownList($model['type'])) ?>
        </div>
        <div class="col-sm-6">
            <?= $form->field($model, 'area_id')->dropDownList(Area::getDropDownList($model['type'])) ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-6">
            <?= $form->field($model, 'region_id')->dropDownList(\frontend\modules\location\models\Region::getDropDownList()) ?>
        </div>
        <div class="col-sm-6">
            <?= $form->field($modelLang, 'address')->textInput(['maxlength' => true]) ?>
        </div>
    </div>
    <?php
    echo $form->field($model, 'coordinates')->widget(MapInputWidget::class,
        [
            'key' => 'AIzaSyCUmy-9mReTZS50UTtYAdxev22BUXYk_ig',
            'latitude' => 48.5,
            'longitude' => 32,
            'zoom' => 6,
            'height' => '600px',
        ]);
    ?>
    <div class="row form-group">
        <div class="col-md-6">
            <?= $form->field($model, 'electric_power')->textInput() ?>
            <div>
                <?= Yii::t('front', 'Зазначте потужність в МВт (наприклад, 10.0). У разі відсутності генерації електроенергії, вкажіть 0.') ?>
            </div>
        </div>
        <div class="col-md-6">
            <?= $form->field($model, 'heat_capacity')->textInput() ?>
            <div>
                <?= Yii::t('front', 'Зазначте потужність в МВт (наприклад, 10.0). У разі відсутності генерації тепла, вкажіть 0.') ?>
            </div>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-md-6">
            <?= $form->field($modelLang, 'initiator')->textInput(['maxlength' => true]) ?>
        </div>
        <div class="col-md-6">
            <?= $form->field($model, 'type_of_promoter')->dropDownList(Item::type_of_promoterKeyRange()) ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'contact_details_of_project_promoter')->textarea() ?>
            <div>
                <?= Yii::t('front', 'Зазначте контактну особу, телефон, адресу електронної пошти, адресу та веб-сайт ініціатора/власника проекту') ?>
            </div>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-md-6">
            <?= $form->field($model, 'date_of_entry')->textInput(['maxlength' => true]) ?>
            <div>
                <?= Yii::t('front', 'Вкажіть дату заповнення цієї форми (наприклад, 10.01.2018)') ?>
            </div>
        </div>
        <div class="col-md-6">
            <?= $form->field($model, 'current_stage_of_project_development')->dropDownList(Item::current_stage_of_project_developmentKeyRange()) ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'description')->textarea() ?>
            <div>
                <?= Yii::t('front', 'Надайте короткий опис проекту: суть, мета проекту, місце розташування, статус реалізації та іншу основну інформацію за проектом.') ?>
            </div>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-md-6">
            <?= $form->field($modelLang, 'annual_power_production')->textInput(['maxlength' => true]) ?>
            <div>
                <?= Yii::t('front', 'У разі відсутності генерації електроенергії, вкажіть 0.') ?>
            </div>
        </div>
        <div class="col-md-6">
            <?= $form->field($modelLang, 'annual_heat_production')->textInput(['maxlength' => true]) ?>
            <div>
                <?= Yii::t('front', 'У разі відсутності генерації тепла, вкажіть 0.') ?>
            </div>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-md-6">
            <?= $form->field($modelLang, 'power_price')->textInput(['maxlength' => true]) ?>
        </div>
        <div class="col-md-6">
            <?= $form->field($modelLang, 'heat_price')->textInput(['maxlength' => true]) ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-md-6">
            <?= $form->field($modelLang, 'project_implementation_period')->textInput(['maxlength' => true]) ?>
        </div>
        <div class="col-md-6">
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'project_documentation')->textarea() ?>
            <div>
                <?= Yii::t('front', 'Вкажіть основні проектні документи в наявності, отримані чи розроблені ініціатором/власником проекту (наприклад, техніко-економічне обґрунтування, бізнес-план, фінансова модель, звіт про аудит та ін.).') ?>
            </div>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'investment_schedule')->textarea() ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?php $drop = Yii::$app->language == 'en-EN' ?
                \frontend\modules\map\models\ItemLang::licensesRangeListEN()
                :
                \frontend\modules\map\models\ItemLang::licensesRangeListUA() ?>
            <?= $form->field($modelLang, 'licenses')->widget(\kartik\select2\Select2::classname(), [
                'data' => $drop,
                'options' => [
//                    'value' => '',
//                    'placeholder' => '',
                    'multiple' => true,
                    'allowClear' => true
                ],
            ]) ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'licenses_other')->textarea() ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'estimated_return_on_investment')->textarea() ?>
            <div>
                <?= Yii::t('front', 'Please describe the expected income sources, including sale of power to [name of power off-taker], sale of heat to [name of heat off-taker], government incentives [specify], other') ?>
            </div>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'technical_parameters')->textarea() ?>
            <div>
                <?= Yii::t('front', 'Вкажіть основне обладнання, яке планується застосувати в проекті, та країну-виробника.') ?>
            </div>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-md-6">
            <?= $form->field($model, 'investment')->textInput(['maxlength' => true]) ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'forms_cooperation_with_investor')->textarea() ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-md-6">
            <?= $form->field($model, 'share_of_project_equity_offered') ?>
            <div>
                <?= Yii::t('front', 'Вкажіть частку співфінансування проекту від 0 до 100%, або визначається шляхом переговорів.') ?>
            </div>
        </div>
        <div class="col-md-6">
            <?= $form->field($modelLang, 'expected_financial_return') ?>
            <div>
                <?= Yii::t('front', 'Вкажіть термін окупності, років, та/або IRR, % протягом 10-річного періоду.') ?>
            </div>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'financing_sources')->textarea() ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'financing_debt')->textarea() ?>
        </div>
    </div>
    <div class="row form-group">
        <div class="col-sm-12">
            <?= $form->field($modelLang, 'expenditure_additional_infrastructure')->textarea() ?>
        </div>
    </div>
    <div class="row form-group">
        <?= (1) ?
            $form->field($model, 'gallery_image')->imageSeveral(
                ['initialPreview' => $model->getGalleryLink(), 'maxFileCount' => 25],
                ['maxFileCount' => 25])
            . $form->field($model, 'gallery_file')->fileSeveral(
                ['initialPreview' => $model->getGalleryFileLink(), 'maxFileCount' => 25],
                ['maxFileCount' => 25])
            : '' ?>
    </div>

    <div class="row form-group">
        <div class="col-sm-6">
            <?= $form->field($model, 'web_site')->textInput(['maxlength' => true]) ?>
        </div>
        <div class="col-sm-6">
            <?= $form->field($model, 'youtube_link')->textInput(['maxlength' => true]) ?>
        </div>
    </div>

    <?php
    if ($model->isNewRecord) {
//        echo $form->field($model, 'type')->hiddenInput()->label('');
//        echo $form->field($model, 'user_id')->hiddenInput(['value' => Yii::$app->getUser()->getId()])->label('');
    }
    //
    echo Html::submitButton($model->isNewRecord ? Yii::t('app', 'Add') : Yii::t('app', 'Save'), ['class' => 'btn btn-info']);
    //
    ActiveForm::end();
    ?>
</div>
<style>
    .select2-container--krajee {
        height: auto;
    }

    .kv-file-zoom {
        display: none;
    }
</style>
