<?php

use yii\db\Migration;
use thread\modules\seo\modules\info\Info as ParentModule;

/**
 * Class m170506_105637_add_table_seo_info_data
 *
 * @author FilamentV <vortex.filament@gmail.com>
 * @copyright (c), Thread
 */
class m170506_105637_add_table_seo_info_data extends Migration
{

    /**
     * Table name
     *
     * @var string
     */
    public $table = '{{%seo_info}}';

    /**
     * Table lang table name
     *
     * @var string
     */
    public $tableLang = '{{%seo_info_lang}}';

    /**
     *
     */
    public function init()
    {
        $this->db = ParentModule::getDb();
        parent::init();
    }

    /**
     *
     */
    public function safeUp()
    {
        /** Insert table */
        $this->batchInsert(
            $this->table,
            [
                'id',
                'alias',
                'default_title',
                'created_at',
                'updated_at',
                'published',
                'deleted',
            ],
            [
                [
                    'id' => 1,
                    'alias' => 'base-url',
                    'default_title' => 'base full url to site',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 2,
                    'alias' => 'web-site-logo',
                    'default_title' => 'full url to logo',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 3,
                    'alias' => 'web-site-name',
                    'default_title' => '',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 4,
                    'alias' => 'web-site-alternatename',
                    'default_title' => '',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 5,
                    'alias' => 'web-site-search-action',
                    'default_title' => '',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 6,
                    'alias' => 'web-site-telephone',
                    'default_title' => 'web site telephone +1-401-555-1212',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 7,
                    'alias' => 'seo-pre-title',
                    'default_title' => 'seo-pre-title',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 8,
                    'alias' => 'seo-post-title',
                    'default_title' => 'seo-post-title',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 9,
                    'alias' => 'seo-pre-description',
                    'default_title' => 'seo-pre-description',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
                [
                    'id' => 10,
                    'alias' => 'seo-post-description',
                    'default_title' => 'seo-post-description',
                    'created_at' => time(),
                    'updated_at' => time(),
                    'published' => '1',
                    'deleted' => '0',
                ],
            ]
        );

        /** Insert table lang */
        $this->batchInsert(
            $this->tableLang,
            [
                'rid',
                'lang',
                'title',
                'value',
            ],
            [
                [
                    'rid' => 1,
                    'lang' => 'en-EN',
                    'title' => 'base full url to site',
                    'value' => 'https://query.example.com',
                ],
                [
                    'rid' => 2,
                    'lang' => 'en-EN',
                    'title' => 'full url to logo',
                    'value' => 'https://query.example.com/logo.png',
                ],
                [
                    'rid' => 3,
                    'lang' => 'en-EN',
                    'title' => 'web site name',
                    'value' => 'web site name',
                ],
                [
                    'rid' => 4,
                    'lang' => 'en-EN',
                    'title' => 'web site alternate name',
                    'value' => 'web site alternate name',
                ],
                [
                    'rid' => 5,
                    'lang' => 'en-EN',
                    'title' => 'https://query.example.com/search?q={search_term_string}',
                    'value' => 'https://query.example.com/search?q={search_term_string}',
                ],
                [
                    'rid' => 6,
                    'lang' => 'en-EN',
                    'title' => 'web site telephone +1-401-555-1212',
                    'value' => '+1-401-555-1212',
                ],
                [
                    'rid' => 7,
                    'lang' => 'en-EN',
                    'title' => 'seo-pre-title',
                    'value' => '',
                ],
                [
                    'rid' => 8,
                    'lang' => 'en-EN',
                    'title' => 'seo-post-title',
                    'value' => '',
                ],
                [
                    'rid' => 9,
                    'lang' => 'en-EN',
                    'title' => 'seo-pre-description',
                    'value' => '',
                ],
                [
                    'rid' => 10,
                    'lang' => 'en-EN',
                    'title' => 'seo-post-description',
                    'value' => '',
                ],
            ]
        );
    }

    /**
     * Cancel migration
     */
    public function safeDown()
    {
        //TODO:: m170506_105637_add_table_seo_info_data
    }
}
