<?php

namespace thread\modules\sys\modules\growl\models;

use Yii;
//
use thread\app\base\models\ActiveRecord;
use thread\modules\sys\modules\growl\Growl as ParentModule;

/**
 * Class Growl
 *
 * @package thread\modules\sys\modules\growl\models
 * @author FilamentV <vortex.filament@gmail.com>
 * @copyright (c), Thread
 */
class Growl extends ActiveRecord
{
    const type_notice = 'notice';
    const type_warning = 'warning';
    const type_danger = 'danger';
    const type_error = 'error';
    const type_success = 'success';
    const type_primary = 'primary';

    /**
     * @return string
     */
    public static function getDb()
    {
        return ParentModule::getDb();
    }

    /**
     * @var
     */
    public static $commonQuery = query\ActiveQuery::class;

    /**
     * @return string
     */
    public static function tableName()
    {
        return '{{%system_growl}}';
    }

    /**
     * @return array
     */
    public function rules()
    {
        return [
            [['user_id', 'message', 'type', 'priority', 'model'], 'required'],
            [['created_at', 'updated_at', 'user_id'], 'integer'],
            [['published', 'deleted', 'is_read'], 'in', 'range' => array_keys(static::statusKeyRange())],
            [['type'], 'in', 'range' => array_keys(static::getTypeRange())],
            ['message', 'string', 'max' => 255],
            ['url', 'string', 'max' => 512],
            ['model', 'string', 'max' => 50],
        ];
    }

    /**
     * @return array
     */
    public function scenarios()
    {
        return [
            'published' => ['published'],
            'deleted' => ['deleted'],
            'is_read' => ['is_read '],
            'backend' => ['type', 'message', 'is_read', 'published', 'deleted', 'model', 'user_id', 'url'],
            'send' => ['type', 'message', 'is_read', 'published', 'deleted', 'model', 'user_id', 'url'],
        ];
    }

    /**
     * @return array
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'message' => Yii::t('app', 'Message'),
            'url' => Yii::t('app', 'Url'),
            'type' => Yii::t('app', 'Type'),
            'is_read' => Yii::t('app', 'Readonly'),
            'user_id' => Yii::t('sys', 'To user'),
            'model' => Yii::t('app', 'Model'),
            'created_at' => Yii::t('app', 'Created at'),
            'updated_at' => Yii::t('app', 'Updated at'),
            'published' => Yii::t('app', 'Published'),
            'deleted' => Yii::t('app', 'Deleted'),
        ];
    }

    /**
     * @return array
     */
    public static function getTypeRange()
    {
        return [
            'notice' => Yii::t('sys', 'notice'),
            'warning' => Yii::t('sys', 'warning'),
//            'error' => Yii::t('growl', 'error'),
            'danger' => Yii::t('sys', 'danger'),
            'success' => Yii::t('sys', 'success'),
            'primary' => Yii::t('sys', 'primary'),
        ];
    }

    /**
     * @param array $ids
     * @param $message
     * @param string $type
     * @param string $url
     * @return mixed
     */
    public static function sendByUserIds(array $ids, $message, $type = 'notice', $url = '', $model = 'growl')
    {
        foreach ($ids as $id) {
            $data[] = [
                $id, $message, $model, $url, $type, self::STATUS_KEY_OFF, time(), time(), self::STATUS_KEY_ON, self::STATUS_KEY_OFF
            ];
        }

        return self::getDb()->createCommand()->batchInsert(self::tableName(),
            ['user_id', 'message', 'model', 'url', 'type', 'is_read', 'created_at', 'updated_at', 'published', 'deleted'],
            $data)->execute();
    }

    /**
     * @return false|string
     */
    public function getDateTime()
    {
        return date('d.m.Y H:i', $this->created_at);
    }
}
