<?php

use yii\db\Migration;
use thread\modules\user\User as ParentModule;

/**
 * Class m160127_022108_create_fv_auth_item_table
 *
 * @author FilamentV <vortex.filament@gmail.com>
 * @copyright (c), Thread
 */
class m160127_022108_create_fv_auth_item_table extends Migration
{
    /**
     * @var string
     */
    public $tableAuthItem = '{{%auth_item}}';

    /**
     * @var string
     */
    public $tableAuthRule = '{{%auth_rule}}';

    /**
     * Core DB connction
     */
    public function init()
    {
        $this->db = ParentModule::getDb();
        parent::init();
    }

    /**
     * Implement migration
     */
    public function safeUp()
    {
        /** Create table */
        $this->createTable($this->tableAuthItem, [
            'name' => $this->string(64)->notNull(),
            'type' => $this->integer(11)->notNull(),
            'description' => $this->text(),
            'rule_name' => $this->string(64)->defaultValue(null),
            'data' => $this->text(),
            'created_at' => $this->integer(11)->defaultValue(null),
            'updated_at' => $this->integer(11)->defaultValue(null)
        ]);

        /** Add PK */
        $this->addPrimaryKey('name', $this->tableAuthItem, 'name');

        /** Add indexes */
        $this->createIndex('rule_name', $this->tableAuthItem, 'rule_name');
        $this->createIndex('idx-auth_item-type', $this->tableAuthItem, 'type');

        /** Create FK to auth rules table */
        $this->addForeignKey(
            'fv_auth_item_ibfk_1',
            $this->tableAuthItem,
            'rule_name',
            $this->tableAuthRule,
            'name',
            'CASCADE',
            'CASCADE'
        );
    }

    /**
     * Cancel migration
     */
    public function safeDown()
    {
        $this->dropForeignKey('fv_auth_item_ibfk_1', $this->tableAuthItem);
        $this->dropIndex('idx-auth_item-type', $this->tableAuthItem);
        $this->dropIndex('rule_name', $this->tableAuthItem);
        $this->dropPrimaryKey('name', $this->tableAuthItem);
        $this->dropTable($this->tableAuthItem);
    }
}
